//--------------------------------------------------------------------
//	
//	SaveWindow.cpp
//
//	Written by: Robert Polic
//	
//	Copyright 1997 Be, Inc. All Rights Reserved.
//	
//--------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#ifndef SAVE_WINDOW_H
#include "SaveWindow.h"
#endif

extern bool		changed;
extern short	track;
extern bool		mute;
extern short	format;
extern bool		quitting;


//====================================================================

TSaveWindow::TSaveWindow(BRect rect, char *title, int cd_id,
						 scsi_toc *toc, short track, BList* list)
			:BWindow(rect, title, B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	BRect		r;

	fTOC = toc;
	fTrack = track;
	fTitleList = list;
	fCDID = cd_id;

	r.Set(PLAY_BUTTON_H, PLAY_BUTTON_V,
		  PLAY_BUTTON_H + SAVE_BUTTON_WIDTH, PLAY_BUTTON_V + SAVE_BUTTON_HEIGHT);
	fPlayButton = new BButton(r, "play", "Preview", new BMessage(PLAY_BUT));
	AddChild(fPlayButton);
	Lock();
	fPlayButton->MakeDefault(TRUE);
	Unlock();

	r.Set(STOP_BUTTON_H, STOP_BUTTON_V,
		  STOP_BUTTON_H + SAVE_BUTTON_WIDTH, STOP_BUTTON_V + SAVE_BUTTON_HEIGHT);
	fStopButton = new BButton(r, "stop", "Stop", new BMessage(STOP_BUT));
	AddChild(fStopButton);
	Lock();
	fStopButton->SetEnabled(FALSE);
	Unlock();

	r.Set(SAVE_BUTTON_H, SAVE_BUTTON_V,
		  SAVE_BUTTON_H + SAVE_BUTTON_WIDTH, SAVE_BUTTON_V + SAVE_BUTTON_HEIGHT);
	fSaveButton = new BButton(r, "save", "Save...", new BMessage(SAVE_BUT));
	AddChild(fSaveButton);

	r.Set(0, 0, SAVE_WIDTH, SAVE_HEIGHT);
	fSaveView = new TSaveView(r, "save", this, fCDID);
	AddChild(fSaveView);
	SetPulseRate(100000.0);
}

//--------------------------------------------------------------------

TSaveWindow::~TSaveWindow()
{
}

//--------------------------------------------------------------------

void TSaveWindow::MessageReceived(BMessage* theMessage)
{
	long				i;
	long				time;
	scsi_play_position	audio;

	switch(theMessage->what) {
		case PLAY_BUT:
			if (fSaveView->fStart != fSaveView->fStop) {
				time = fSaveView->fTrackStart + fSaveView->fStart;
				audio.start_m = time / (60 * 75);
				time %= (60 * 75);
				audio.start_s = time / 75;
				audio.start_f = time % 75;

				time = fSaveView->fTrackStart + fSaveView->fStop;
				audio.end_m = time / (60 * 75);
				time %= (60 * 75);
				audio.end_s = time / 75;
				audio.end_f = time % 75;
				ioctl(fCDID, SCSI_PLAY_POSITION, &audio);
				fSaveView->Pulse();
			}
			break;

		case SAVE_BUT:
			RunSavePanel("");
			break;

		case STOP_BUT:
			if (fCDID) {
				ioctl(fCDID, SCSI_STOP_AUDIO);
				fSaveView->Pulse();
			}
			break;

		case AIFF:
		case RAW:
		case WAVE:
			format = theMessage->what;
			break;

		default:
			if ((theMessage->what > 99) && (theMessage->what < 255)) {
				i = theMessage->what - 100;
				fSaveView->fTrackStart = (fTOC->toc_data[4 + (i * 8) + 5] * 60 * 75) +
										 (fTOC->toc_data[4 + (i * 8) + 6] * 75) +
										 (fTOC->toc_data[4 + (i * 8) + 7]);
				fSaveView->fStart = 0;
				fSaveView->fTrackLength = ((fTOC->toc_data[4 + ((i + 1) * 8) + 5] * 60 * 75) +
										   (fTOC->toc_data[4 + ((i + 1) * 8) + 6] * 75) +
										   (fTOC->toc_data[4 + ((i + 1) * 8) + 7])) -
										   fSaveView->fTrackStart;
				fSaveView->fStop = fSaveView->fTrackLength;
				fSaveView->DrawDiskSpace();
				fSaveView->DrawSelector();
				fTrack = i + 1;
			}
			else
				inherited::MessageReceived(theMessage);
	}
}

//--------------------------------------------------------------------

bool TSaveWindow::QuitRequested()
{
	if (quitting)
		return TRUE;
	else {
		Hide();
		return FALSE;
	}
}

//--------------------------------------------------------------------

void TSaveWindow::SaveRequested(record_ref dir, const char* name)
{
	bool			error = FALSE;
	char			bummer[100] = "Error creating file.";
	char			bummer1[100] = "Not enough disk space on volume to save selection.";
	long			length;
	ulong			type;
	ulong			creator;
	BAlert			*alert;
	BDirectory		theDir;
	BFile			*theFile;
	BRect			r;
	BRect			wind_rect;
	BVolume			*volume;
	TStatusWindow	*statusWindow;

	if (fSaveView->fStart != fSaveView->fStop) {
		theFile = new BFile();
		theDir.SetRef(dir);

		length = (((fSaveView->fStop - fSaveView->fStart) * 2352) + 511) & -512;
		volume = new BVolume(theDir.VolumeID());
		if (volume->Capacity() < length) {
			alert = new BAlert("", bummer1, "Sorry");
			alert->Go();
			delete theFile;
			delete volume;
			return;
		}
		delete volume;

		if (theDir.Create(name, theFile) == B_NO_ERROR) {
			switch (format) {
				case RAW:
					type = 'RAWM';
					creator = 'PLAY';
					break;

				case AIFF:
					type = 'AIFF';
					creator = 'PLAY';
					break;

				case WAVE:
					type = 'WAVE';
					creator = 'PLAY';
					break;

				default:
					break;
			}
			theFile->SetTypeAndApp(type, creator);
			theFile->Open(B_READ_WRITE);
			if (theFile->Error() == B_NO_ERROR) {
				wind_rect = Frame();
				r.left = wind_rect.left + ((wind_rect.Width() - STATUS_WIDTH) / 2);
				r.right = r.left + STATUS_WIDTH;
				r.top = wind_rect.top + ((wind_rect.Height() - STATUS_HEIGHT) / 2);
				r.bottom = r.top + STATUS_HEIGHT;

				statusWindow = new TStatusWindow(r, "Save Progress",
								theFile,
								fSaveView->fTrackStart,
								fSaveView->fStart,
								fSaveView->fStop,
								fCDID);
			}
			else {
				theFile->Close();
				delete theFile;
			}
		}
		else
			error = TRUE;
	}
	else
		error = TRUE;

	if (error) {
		alert = new BAlert("", bummer, "Sorry");
		alert->Go();
	}
}

//--------------------------------------------------------------------

void TSaveWindow::Show(short track)
{
	fTrack = track;
	ResetMenu();
	BWindow::Show();
}

//--------------------------------------------------------------------

void TSaveWindow::ResetMenu()
{
	char		text[100];
	short		i = 0;
	long		track_num;
	BMenuItem	*mitem;
	track_info	*track_data;

	if (fSaveView->fTrackMenu) {
		while (mitem = fSaveView->fTrackMenu->RemoveItem((long)0)) {
			delete mitem;
		}
		while (fTOC->toc_data[4 + (i * 8) + 2] != 0xaa) {
			track_num = fTOC->toc_data[4 + (i * 8) + 2];
			track_data = (track_info*)fTitleList->ItemAt(track_num);
			if (track_data)
				fSaveView->fTrackMenu->AddItem(mitem = new BMenuItem(
					track_data->title, new BMessage(100 + i)));
			else {
				sprintf(text, "Track %d", track_num);
				fSaveView->fTrackMenu->AddItem(mitem = new BMenuItem(text, new BMessage(100 + i)));
			}
			if (fTrack == fTOC->toc_data[4 + (i * 8) + 2]) {
				fSaveView->fTrackStart = (fTOC->toc_data[4 + (i * 8) + 5] * 60 * 75) +
										 (fTOC->toc_data[4 + (i * 8) + 6] * 75) +
										 (fTOC->toc_data[4 + (i * 8) + 7]);
				fSaveView->fStart = 0;
				fSaveView->fTrackLength = ((fTOC->toc_data[4 + ((i + 1) * 8) + 5] * 60 * 75) +
										   (fTOC->toc_data[4 + ((i + 1) * 8) + 6] * 75) +
										   (fTOC->toc_data[4 + ((i + 1) * 8) + 7])) -
										   fSaveView->fTrackStart;
				fSaveView->fStop = fSaveView->fTrackLength;
				mitem->SetMarked(TRUE);
			}
			i++;
		}
	}
}


//====================================================================

TSaveView::TSaveView(BRect rect, char *title, TSaveWindow* wind, int cd_id)
		  :BView(rect, title, B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED)
{
	BMenuBar	*mbar;
	BMenuItem	*mitem;
	BRect		r;
	rgb_color	c;

	fLastx = -1;
	fWindow = wind;
	fCDID = cd_id;
	c.red = c.green = c.blue = 216;
	SetViewColor(c);

	r = Bounds();
	r.left += 3;
	r.right -= 3;
	r.top = SELECTOR_V;
	r.bottom = SELECTOR_V + SELECTOR_HEIGHT;
	r.OffsetTo(0, 0);
	fOffScreen = new BBitmap(r, B_COLOR_8_BIT, TRUE);
	fOffScreen->AddChild(fOffView = new BView(r, "", B_FOLLOW_ALL, B_WILL_DRAW));

	r.Set(TRACK_MENU_H, TRACK_MENU_V,
		  TRACK_MENU_H + TRACK_MENU_WIDTH, TRACK_MENU_V + TRACK_MENU_HEIGHT);
	mbar = new BMenuBar(r, "tracks", B_FOLLOW_LEFT + B_FOLLOW_TOP, B_ITEMS_IN_COLUMN);
	AddChild(mbar);
	fTrackMenu = new BPopUpMenu("Tracks");
	mbar->AddItem(fTrackMenu);

	r.Set(FORMAT_MENU_H, FORMAT_MENU_V,
		  FORMAT_MENU_H + FORMAT_MENU_WIDTH, FORMAT_MENU_V + FORMAT_MENU_HEIGHT);
	mbar = new BMenuBar(r, "format", B_FOLLOW_LEFT + B_FOLLOW_TOP, B_ITEMS_IN_COLUMN);
	AddChild(mbar);
	fFormatMenu = new BPopUpMenu("Format");
	mbar->AddItem(fFormatMenu);
	fFormatMenu->AddItem(mitem = new BMenuItem("Raw", new BMessage(RAW)));
	if (format == RAW)
		mitem->SetMarked(TRUE);
	fFormatMenu->AddSeparatorItem();
	fFormatMenu->AddItem(mitem = new BMenuItem("AIFF", new BMessage(AIFF)));
	if (format == AIFF)
		mitem->SetMarked(TRUE);
	fFormatMenu->AddItem(mitem = new BMenuItem("WAVE", new BMessage(WAVE)));
	if (format == WAVE)
		mitem->SetMarked(TRUE);
	mitem->SetEnabled(FALSE);
}

//--------------------------------------------------------------------

TSaveView::~TSaveView()
{
	delete fOffScreen;
}

//--------------------------------------------------------------------

void TSaveView::AttachedToWindow()
{
}

//--------------------------------------------------------------------

void TSaveView::Draw(BRect where)
{
	BRect	r;
	BRect	r1;
	BRect	r2;

	r1.Set(0, 0, SAVE_BUTTON_WIDTH, SAVE_BUTTON_HEIGHT);
	r = Frame();
	SetHighColor(216, 216, 216);
	FillRect(r);
	r2.Set(PLAY_BUTTON_H, PLAY_BUTTON_V,
		   PLAY_BUTTON_H + SAVE_BUTTON_WIDTH, PLAY_BUTTON_V + SAVE_BUTTON_HEIGHT);
	if (r2.Intersects(where))
		fWindow->fPlayButton->Draw(r1);

	r2.Set(STOP_BUTTON_H, STOP_BUTTON_V,
		   STOP_BUTTON_H + SAVE_BUTTON_WIDTH, STOP_BUTTON_V + SAVE_BUTTON_HEIGHT);
	if (r2.Intersects(where))
		fWindow->fStopButton->Draw(r1);

	r2.Set(SAVE_BUTTON_H, SAVE_BUTTON_V,
		   SAVE_BUTTON_H + SAVE_BUTTON_WIDTH, SAVE_BUTTON_V + SAVE_BUTTON_HEIGHT);
	if (r2.Intersects(where))
		fWindow->fSaveButton->Draw(r1);

	// Window shading
	SetHighColor(255, 255, 255);
	StrokeLine(BPoint(r.left, r.top), BPoint(r.right, r.top));
	StrokeLine(BPoint(r.left, r.top), BPoint(r.left, r.bottom));
	StrokeLine(BPoint(r.left + 8, DIVIDER + 1), BPoint(r.right - 8, DIVIDER + 1));
	SetHighColor(152, 152, 152);
	StrokeLine(BPoint(r.right, r.top + 1), BPoint(r.right, r.bottom));
	StrokeLine(BPoint(r.left + 1, r.bottom), BPoint(r.right, r.bottom));
	StrokeLine(BPoint(r.left + 8, DIVIDER), BPoint(r.right - 8, DIVIDER));

	SetHighColor(0, 0, 0);
	SetFontName("Emily");
	SetFontSize(12);
	SetDrawingMode(B_OP_OVER);
	MovePenTo(TRACK_TEXT_H, TRACK_TEXT_V);
	DrawString("Track:");
	MovePenTo(FORMAT_TEXT_H, FORMAT_TEXT_V);
	DrawString("File Format:");
	MovePenTo(DISK_TEXT_H, DISK_TEXT_V);
	DrawString("Disk Space:");

	r.Set(DISK_SPACE_H, DISK_TEXT_V - 13,
		  DISK_SPACE_H + 50, DISK_TEXT_V);
	if (r.Intersects(where))
		DrawDiskSpace();

	r = Bounds();
	r.left += 3;
	r.right -= 3;
	r.top = SELECTOR_V;
	r.bottom = SELECTOR_V + SELECTOR_HEIGHT;
	if (r.Intersects(where))
		DrawSelector();
}

//--------------------------------------------------------------------

void TSaveView::MouseDown(BPoint thePoint)
{
	bool	tracking_start = FALSE;
	ulong	buttons;
	float	temp;
	float	unit;
	float	width;
	float	offset;
	BPoint	where;
	BRect	back;
	BRect	track;

	back = Bounds();
	back.left += 8;
	back.right -= 8;
	back.top = SELECTOR_V + 15;
	back.bottom = back.top + 7;

	unit = (back.right - back.left) / fTrackLength;
	track = back;
	track.left += (fStart * unit) - 5;
	track.right = track.left + 10;
	track.bottom += 14;
	if ((track.Contains(thePoint)) || ((back.Contains(thePoint)) &&
		(thePoint.x < track.left))) {	
		tracking_start = TRUE;
		goto track_mouse;
	}

	track.left = back.left + (fStop * unit) - 5;
	track.right = track.left + 10;
	if ((track.Contains(thePoint)) || ((back.Contains(thePoint)) &&
		(thePoint.x > track.right)))
		goto track_mouse;
	else
		return;

track_mouse:
	if (track.Contains(thePoint))
		offset = thePoint.x - (track.left + 5);
	else
		offset = 0.0;
	do {
		GetMouse(&where, &buttons);
		where.x -= offset;
		if (where.x < back.left)
			where.x = back.left;
		if (where.x > back.right)
			where.x = back.right;
		temp = (where.x - back.left) * (1 / unit);
		if (tracking_start) {
			if (temp > fStop)
				temp = fStop;
			if ((int)temp != (int)fStart) {
				fStart = temp;
				DrawDiskSpace();
				DrawSelector();
			}
		}
		else {
			if (temp < fStart)
				temp = fStart;
			if ((int)temp != (int)fStop) {
				fStop = temp;
				DrawDiskSpace();
				DrawSelector();
			}
		}
		Pulse();
	} while (buttons);
}

//--------------------------------------------------------------------

void TSaveView::Pulse()
{
	long				position;
	float				unit;
	BRect				back;
	scsi_position		pos;

	if ((fCDID) && (!Window()->IsHidden())) {
		if (changed) {
			changed = FALSE;
			fWindow->fTrack = track;
			fWindow->ResetMenu();
			DrawSelector();
			DrawDiskSpace();
		}

		ioctl(fCDID, SCSI_GET_POSITION, &pos);
		if (pos.position[1] == 0x11) {
			if (!fWindow->fStopButton->IsEnabled()) {
				fWindow->Lock();
				fWindow->fStopButton->SetEnabled(TRUE);
				fWindow->Unlock();
			}
		}
		else if (fWindow->fStopButton->IsEnabled()) {
			fWindow->Lock();
			fWindow->fStopButton->SetEnabled(FALSE);
			fWindow->Unlock();
		}

		if (((pos.position[1] == 0x11) ||
			((pos.position[1] == 0x12) && (pos.position[6]))) &&
			 (pos.position[6] == fWindow->fTrack)) {
			back = Bounds();
			back.left += 8;
			back.right -= 8;

			unit = (back.right - back.left) / fTrackLength;
			position = (pos.position[13] * 60 * 75) + (pos.position[14] * 75) +
					pos.position[15];
			if (position != fPos) {
				fPos = position;
				position *= unit;
				fLastx = (int)position;
				DrawSelector();
			}
		}
		else if (fLastx >= 0) {
			fLastx = -1;
			DrawSelector();
		}
	}
}

//--------------------------------------------------------------------

void TSaveView::DrawDiskSpace()
{
	char	text[256];
	float	length;
	BRect	r;

	SetFontName("Emily");
	SetFontSize(12);
	// length = ((((fStop - fStart) * 2352) + 511) & -512) / 1024;
	length = ((((fStop - fStart) * 2352) + 511) & -512) / 1024.0;
	if (length > 1024) {
		length /= 1024;
		sprintf(text, "%d.%1d MB", (int)length, (int)((length - (int)length) * 10));
	}
	else
		sprintf(text, "%d KB", (int)length);
	r.Set(DISK_SPACE_H, DISK_TEXT_V - 13,
		  DISK_SPACE_H + 50, DISK_TEXT_V);
	SetHighColor(216, 216, 216);
	FillRect(r);
	SetHighColor(0, 0, 0);
	SetDrawingMode(B_OP_OVER);
	MovePenTo(DISK_SPACE_H, DISK_TEXT_V);
	DrawString(text);
}

//--------------------------------------------------------------------

void TSaveView::DrawSelector()
{
	char	text[256];
	long	min;
	long	sec;
	long	frame;
	long	time;
	float	start_width;
	float	stop_width;
	float	unit;
	float	l;
	BRect	sr;
	BRect	dr;

	fOffScreen->Lock();
	fOffView->SetFontName("Erich");
	fOffView->SetFontSize(9);
	sr = fOffView->Bounds();
	fOffView->SetHighColor(216, 216, 216);
	fOffView->FillRect(sr);

	fOffView->SetHighColor(128, 128, 128);
	fOffView->StrokeLine(BPoint(sr.left + 4, sr.top + 14),
						 BPoint(sr.right - 5, sr.top + 14));
	fOffView->StrokeLine(BPoint(sr.left + 4, sr.top + 14),
						 BPoint(sr.left + 4, sr.top + 23));
	fOffView->SetHighColor(255, 255, 255);
	fOffView->StrokeLine(BPoint(sr.left + 5, sr.top + 23),
						 BPoint(sr.right - 4, sr.top + 23));
	fOffView->StrokeLine(BPoint(sr.right - 4, sr.top + 23),
						 BPoint(sr.right - 4, sr.top + 14));
	dr.left = sr.left + 5;
	dr.top = sr.top + 15;
	dr.right = sr.right - 5;
	dr.bottom = dr.top + 7;
	fOffView->SetHighColor(184, 184, 184);
	fOffView->FillRect(dr);

	unit = (dr.right - dr.left) / fTrackLength;
	dr.right = dr.left + (fStop * unit);
	dr.left += fStart * unit;
	fOffView->SetHighColor(51, 0, 203);
	fOffView->SetHighColor(102, 152, 203);
	fOffView->FillRect(dr);

	fOffView->SetHighColor(80, 80, 80);
	// Draw start indicator outline
	fOffView->StrokeLine(BPoint(dr.left, dr.top + 6),
						 BPoint(dr.left - 5, dr.top + 11));
	fOffView->StrokeLine(BPoint(dr.left - 5, dr.top + 11),
						 BPoint(dr.left + 5, dr.top + 11));
	fOffView->StrokeLine(BPoint(dr.left + 5, dr.top + 11),
						 BPoint(dr.left, dr.top + 6));

	// Draw stop indicator outline
	fOffView->StrokeLine(BPoint(dr.right, dr.top + 6),
						 BPoint(dr.right - 5, dr.top + 11));
	fOffView->StrokeLine(BPoint(dr.right - 5, dr.top + 11),
						 BPoint(dr.right + 5, dr.top + 11));
	fOffView->StrokeLine(BPoint(dr.right + 5, dr.top + 11),
						 BPoint(dr.right, dr.top + 6));

	// Draw current position indicator outline
	if (fLastx >= 0) {
		fOffView->StrokeLine(BPoint(5 + fLastx, dr.top + 1),
							 BPoint(5 + fLastx - 5, dr.top - 4));
		fOffView->StrokeLine(BPoint(5 + fLastx - 5, dr.top - 4),
							 BPoint(5 + fLastx + 5, dr.top - 4));
		fOffView->StrokeLine(BPoint(5 + fLastx + 5, dr.top - 4),
							 BPoint(5 + fLastx, dr.top + 1));
	}

	// Fill start indicator
	fOffView->SetHighColor(0, 255, 0);
	fOffView->StrokeLine(BPoint(dr.left, dr.top + 7),
						 BPoint(dr.left, dr.top + 7));
	fOffView->StrokeLine(BPoint(dr.left - 1, dr.top + 8),
						 BPoint(dr.left + 1, dr.top + 8));
	fOffView->StrokeLine(BPoint(dr.left - 2, dr.top + 9),
						 BPoint(dr.left + 2, dr.top + 9));
	fOffView->StrokeLine(BPoint(dr.left - 3, dr.top + 10),
						 BPoint(dr.left + 3, dr.top + 10));

	// Draw start time
	fOffView->SetHighColor(0, 105, 0);
	time = fStart;
	min = time / (60 * 75);
	time %= (60 * 75);
	sec = time / 75;
	time %= 75;
	frame = (time / 75.0) * 1000.0;
	sprintf(text, "%.2d:%.2d.%.3d", min, sec, frame);
	start_width = fOffView->StringWidth(text);
	if (((dr.left - 4) + start_width) > sr.right)
		l = sr.right - start_width;
	else
		l = dr.left - 4;
	fOffView->MovePenTo(l, dr.top + 23);
	fOffView->SetDrawingMode(B_OP_OVER);
	fOffView->DrawString(text);
	fOffView->SetDrawingMode(B_OP_COPY);

	// Fill stop indicator
	fOffView->SetHighColor(255, 0, 0);
	fOffView->StrokeLine(BPoint(dr.right, dr.top + 7),
						 BPoint(dr.right, dr.top + 7));
	fOffView->StrokeLine(BPoint(dr.right - 1, dr.top + 8),
						 BPoint(dr.right + 1, dr.top + 8));
	fOffView->StrokeLine(BPoint(dr.right - 2, dr.top + 9),
						 BPoint(dr.right + 2, dr.top + 9));
	fOffView->StrokeLine(BPoint(dr.right - 3, dr.top + 10),
						 BPoint(dr.right + 3, dr.top + 10));

	// Draw stop time
	time = fStop;
	min = time / (60 * 75);
	time %= (60 * 75);
	sec = time / 75;
	time %= 75;
	frame = (time / 75.0) * 1000.0;
	sprintf(text, "%.2d:%.2d.%.3d", min, sec, frame);
	stop_width = fOffView->StringWidth(text);
	if (((dr.right - 4) + stop_width) > sr.right)
		l = sr.right - stop_width;
	else
		l = dr.right - 4;
	if ((dr.left + start_width + 4) > l)
		fOffView->MovePenTo(l, dr.top + 34);
	else
		fOffView->MovePenTo(l, dr.top + 23);
	fOffView->SetDrawingMode(B_OP_OVER);
	fOffView->DrawString(text);
	fOffView->SetDrawingMode(B_OP_COPY);

	// Fill current position indicator
	if (fLastx >= 0) {
		fOffView->SetHighColor(0, 0, 255);
		fOffView->StrokeLine(BPoint(5 + fLastx, dr.top),
							 BPoint(5 + fLastx, dr.top));
		fOffView->StrokeLine(BPoint(5 + fLastx - 1, dr.top - 1),
							 BPoint(5 + fLastx + 1, dr.top - 1));
		fOffView->StrokeLine(BPoint(5 + fLastx - 2, dr.top - 2),
							 BPoint(5 + fLastx + 2, dr.top - 2));
		fOffView->StrokeLine(BPoint(5 + fLastx - 3, dr.top - 3),
							 BPoint(5 + fLastx + 3, dr.top - 3));

		// Draw current position time
		fOffView->SetHighColor(51, 51, 102);
		time = fPos;
		min = time / (60 * 75);
		time %= (60 * 75);
		sec = time / 75;
		time %= 75;
		frame = (time / 75.0) * 1000.0;
		sprintf(text, "%.2d:%.2d.%.3d", min, sec, frame);
		start_width = fOffView->StringWidth(text);
		if (((5 + fLastx - 4) + start_width) > sr.right)
			l = sr.right - start_width;
		else
			l = 5 + fLastx - 4;
		fOffView->MovePenTo(l, dr.top - 8);
		fOffView->SetDrawingMode(B_OP_OVER);
		fOffView->DrawString(text);
		fOffView->SetDrawingMode(B_OP_COPY);
	}

	fOffView->Sync();
	DrawBitmap(fOffScreen, BPoint(SELECTOR_H, SELECTOR_V));
	fOffScreen->Unlock();
}

//====================================================================

TStatusWindow::TStatusWindow(BRect rect, char *title, BFile *theFile,
				long trackStart, long start, long stop, int cd_id)
			  :BWindow(rect, title, B_TITLED_WINDOW, B_NOT_RESIZABLE |
													 B_NOT_ZOOMABLE |
													 B_NOT_CLOSABLE)
{
	BButton			*but;
	BCheckBox		*box;
	BRect			r;
	scsi_inquiry	info;

	fStop = FALSE;
	fCDID = cd_id;

	r.left = (STATUS_WIDTH - STATUS_BUTTON_WIDTH) - 10;
	r.right = r.left + STATUS_BUTTON_WIDTH;
	r.top = STATUS_HEIGHT - (STATUS_BUTTON_HEIGHT + 9);
	r.bottom = r.top + STATUS_BUTTON_HEIGHT;
	but = new BButton(r, "status", "Stop", new BMessage(STATUS_BUTTON));
	AddChild(but);

	r.top -= STATUS_MUTE_HEIGHT + 9;
	r.bottom = r.top + STATUS_MUTE_HEIGHT;
	r.right = r.left + STATUS_MUTE_WIDTH;
	box = new BCheckBox(r, "status", "Mute", new BMessage(STATUS_MUTE));
	AddChild(box);

	// If it's not a Toshiba 3501 or 3601 always mute
	if ((ioctl(fCDID, SCSI_INQUIRY, &info) != B_NO_ERROR) || 
		(info.inquiry_data[26] != '3') ||
		((info.inquiry_data[27] != '6') && (info.inquiry_data[27] != '5') &&
		 (info.inquiry_data[27] != '7'))) {
		Lock();
		mute = TRUE;
		box->SetEnabled(FALSE);
		Unlock();
	}
	if (mute) {
		Lock();
		box->SetValue(1);
		Unlock();
	}

	r.Set(0, 0, STATUS_WIDTH, STATUS_HEIGHT);
	fView = new TStatusView(r, "Status", theFile, trackStart,
							start, stop, this, fCDID);
	AddChild(fView);
	Show();
}

//--------------------------------------------------------------------

void TStatusWindow::MessageReceived(BMessage* theMessage)
{
	if (theMessage->what == STATUS_BUTTON)
		fStop = TRUE;
	else
		if (theMessage->what == STATUS_MUTE) {
			if (mute)
				mute = FALSE;
			else
				mute = TRUE;
		}
}

//--------------------------------------------------------------------

TStatusView::TStatusView(BRect rect, char *title, BFile *theFile,
						 long trackStart, long start, long stop,
						 TStatusWindow *wind, int cd_id)
			:BView(rect, title, B_FOLLOW_ALL, B_WILL_DRAW)
{
	fFile = theFile;
	fTrackStart = trackStart;
	fStart = start;
	fStop = stop;
	fWind = wind;
	fCDID = cd_id;
}

//--------------------------------------------------------------------

void TStatusView::AttachedToWindow()
{
	thread_id	thread;

	thread = spawn_thread(this->SaveThread, "SAVE", B_DISPLAY_PRIORITY, this);
	resume_thread(thread);
}

//--------------------------------------------------------------------

void TStatusView::Draw(BRect where)
{
	BRect	r;

	SetHighColor(0, 0, 0);
	SetFontName("Emily");
	SetFontSize(12);
	MovePenTo(STATUS_TEXT_H, STATUS_TEXT_V);
	DrawString("Saving Track");

	r.Set(PROGRESS_H, PROGRESS_V, PROGRESS_H + PROGRESS_WIDTH,
			PROGRESS_V + PROGRESS_HEIGHT);
	StrokeRect(r);
}

//--------------------------------------------------------------------

long TStatusView::SaveThread(void *arg)
{
	bool			tossed;
	char			bummer1[100] = "A disk error occurred saving the track.";
	char			bummer2[100] = "A error occurred reading the track.";
	short			tmp;
	short			tmp1;
	short			tmp2;
	short			result;
	short			tmp_prev_l;
	short			tmp_prev_r;
	uchar*			data;
	uchar*			ptr1;
	uchar*			ptr2;
	int				loop;
	int				i;
	ulong			buf_size;
	float			orig_length;
	float			temp;
	ulong			length;
	ulong			start;
	ulong			frames;
	long			bytes;
	long			time;
	float			width;
	BAlert			*error;
	BRect			r;
	scsi_read_cd	audio;
	ContainerChunk	form;
	CommonChunk		common;
	SoundDataChunk	sound;
	TStatusView		*myView = (TStatusView *)arg;

	r.Set(PROGRESS_H + 1, PROGRESS_V + 1, 0,
		 (PROGRESS_V + 1) + (PROGRESS_HEIGHT - 2));
	width = PROGRESS_WIDTH - 2;

	length = myView->fStop - myView->fStart;
	orig_length = length;
	form.ckID = FORMID;
	form.ckSize =
		sizeof(ContainerChunk) - sizeof(ChunkHeader) +
		sizeof(CommonChunk) + sizeof(SoundDataChunk) +
		(length * 2352);
		form.formType = AIFFID;
	if (format == AIFF)
		myView->fFile->Write(&form, sizeof(ContainerChunk));

	common.ckID = CommonID;
	common.ckSize = sizeof(CommonChunk) - sizeof(ChunkHeader);
	common.numChannels = 2;
	common.numSampleFrames = length * (2352 / 4);
	common.sampleSize = 16;
	common.sampleRate = 0x400eac44;
	for (i = 0; i < sizeof(common.sampleRate1); i++)
		common.sampleRate1[i] = 0;
	if (format == AIFF)
		myView->fFile->Write(&common, sizeof(CommonChunk));

	sound.ckID = SoundDataID;
	sound.ckSize =
	sizeof(SoundDataChunk) - sizeof(ChunkHeader) + (length * 2352);
	sound.offset = 0;
	sound.blockSize = 0;
	if (format == AIFF)
		myView->fFile->Write(&sound, sizeof(SoundDataChunk));

	// grab 1 seconds of audio (or less) each pass
	frames = min(1 * 75, length);
	buf_size = frames * 2352;
	data = (uchar *)malloc(buf_size);
	audio.buffer = (char *)data;
	start = myView->fStart + myView->fTrackStart;

	while ((length > 0) && (!myView->fWind->fStop)) {
		time = start;
		audio.start_m = time / (60 * 75);
		time %= (60 * 75);
		audio.start_s = time / 75;
		audio.start_f = time % 75;

		time = min(frames, length);
		audio.length_m = time / (60 * 75);
		time %= (60 * 75);
		audio.length_s = time / 75;
		audio.length_f = time % 75;

		audio.buffer_length = min(frames, length) * 2352;

		if (mute)
			audio.play = FALSE;
		else
			audio.play = TRUE;

		if (ioctl(myView->fCDID, SCSI_READ_CD, &audio) != B_NO_ERROR) {
			error = new BAlert("", bummer2, "Sorry");
			error->Go();
			goto exit;
		}

		ptr1 = ptr2 = data;
		bytes = 0;
		for (i = 0; i < audio.buffer_length; i += 2) {
			tossed = FALSE;
			tmp = (short)((ptr2[1] << 8) | ptr2[0]);
			if ((0) && (i > 2) && (audio.buffer_length - i >= 98)) {
				if (i & 0x02)
					tmp1 = tmp_prev_r;
				else
					tmp1 = tmp_prev_l;
				if (tmp > tmp1)
					result = tmp - tmp1;
				else
					result = tmp1 - tmp;
				if (result > 5000) {
					tmp1 = (short)((ptr2[95] << 8) | ptr2[94]);
					tmp2 = (short)((ptr2[99] << 8) | ptr2[98]);
					if (tmp1 > tmp2)
						result = tmp1 - tmp2;
					else
						result = tmp2 - tmp1;
					if (result > 5000) {
						printf("tossing data\n (%d)", result);
						tossed = TRUE;
						ptr2 += 96;
						i += 96;
					}
				}
			}
			if (!tossed) {
				ptr2 += 2;
				if (i & 0x02)
					tmp_prev_r = tmp;
				else
					tmp_prev_l = tmp;
				*(short *)(&ptr1[0]) = ((long)tmp * 7) / 10;
				ptr1 += 2;
				bytes += 2;
			}
		}
		if (bytes) {
			myView->fFile->Write(data, bytes);
			if (myView->fFile->Error() != B_NO_ERROR) {
				error = new BAlert("", bummer1, "Sorry");
				error->Go();
				goto exit;
			}
		}

		start += min(frames, length);
		length -= min(frames, length);

		temp = orig_length - length;
		r.right = r.left + ((temp / orig_length) * width);
		myView->fWind->Lock();
		myView->SetHighColor(128, 128, 128);
		myView->FillRect(r);
		myView->fWind->Unlock();
		snooze(1);
	}
exit:;
	free(data);
	myView->fFile->Close();
	delete myView->fFile;
	myView->fWind->Quit();
	return B_NO_ERROR;
}
