//--------------------------------------------------------------------
//	
//	TrackView.cpp
//
//	Written by: Robert Polic
//	
//	Copyright 1997 Be, Inc. All Rights Reserved.
//	
//--------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>

#ifndef TRACK_VIEW_H
#include "TrackView.h"
#endif


//====================================================================

TTrackView::TTrackView(BRect rect, char *title, scsi_toc *toc, BList *list,
					   BWindow *wind, int cd_id)
	   :BView(rect, title, B_FOLLOW_NONE, B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE | B_PULSE_NEEDED)
{
	fTOC = toc;
	fList = list;
	fWindow = wind;
	fCDID = cd_id;
}

//--------------------------------------------------------------------

TTrackView::~TTrackView()
{
	if (fTextView)
		KillEditField(TRUE);
}

//--------------------------------------------------------------------

void TTrackView::AttachedToWindow()
{
	rgb_color	c;

	fTextView = NIL;
	fTrack = 0;
	c.red = c.green = c.blue = 216;
	SetViewColor(c);
	SetFontName("Erich");
	SetFontSize(12);
}

//--------------------------------------------------------------------

void TTrackView::Draw(BRect where)
{
	long		index = 1;
	BRect		r;
	track_info	*track_data;

	r = Bounds();
	SetHighColor(255, 255, 255);
	StrokeLine(BPoint(r.left, r.top), BPoint(r.right, r.top));
	StrokeLine(BPoint(r.left, r.top), BPoint(r.left, r.bottom));
	SetHighColor(184, 184, 184);
	StrokeLine(BPoint(r.right, r.top + 1), BPoint(r.right, r.bottom));
	StrokeLine(BPoint(r.left + 1, r.bottom), BPoint(r.right, r.bottom));

	SetHighColor(0, 0, 0);
	SetDrawingMode(B_OP_OVER);
	MovePenTo(TITLE_POS + 3, HEADER - 4);
	DrawString("Title");
	MovePenTo(TIME_POS + 3, HEADER - 4);
	DrawString("Time");
	SetDrawingMode(B_OP_COPY);
	fTracks = 0;
	while (track_data = (track_info*)fList->ItemAt(index)) {
		fTracks++;
		SetHighColor(128, 128, 128);
		StrokeLine(BPoint(MARGIN, HEADER + ((index - 1) * CELL_HEIGHT)),
				   BPoint(MARGIN + CELL_WIDTH, HEADER + ((index - 1) * CELL_HEIGHT)));
		StrokeLine(BPoint(MARGIN, HEADER + ((index - 1) * CELL_HEIGHT)),
				   BPoint(MARGIN, HEADER + index * CELL_HEIGHT));
		SetHighColor(255, 255, 255);
		StrokeLine(BPoint(MARGIN + 1, HEADER + index * CELL_HEIGHT - 1),
				   BPoint(MARGIN + CELL_WIDTH, HEADER + index * CELL_HEIGHT - 1));
		StrokeLine(BPoint(MARGIN + CELL_WIDTH, HEADER + ((index - 1) * CELL_HEIGHT + 1)),
				   BPoint(MARGIN + CELL_WIDTH, HEADER + index * CELL_HEIGHT - 1));
		if (fTracks == fTrack)
			DrawTrack(index, TRUE);
		else
			DrawTrack(index, FALSE);
		index++;
	}

	if (fTextView) {
		r.Set(TITLE_POS - 1, HEADER + (CELL_HEIGHT * fField) + 1,
			  TIME_POS - 8, HEADER + (CELL_HEIGHT * (fField + 1) - 3));
		if (r.Intersects(where)) {
			r.Set(where.left - (TITLE_POS - 1),
				  where.top - (HEADER + (CELL_HEIGHT * fField) + 1),
				  where.right - (TIME_POS - 8),
				  where.bottom - (HEADER + (CELL_HEIGHT * (fField + 1) - 3)));
			fTextView->FillRect(r, B_SOLID_LOW);
			fTextView->Draw(r);
		}
	}
}

//--------------------------------------------------------------------

void TTrackView::MouseDown(BPoint thePoint)
{
	long	index;
	BRect	r;
	BRect	r1;

	r1 = Bounds();
	r.Set(TITLE_POS, HEADER, TIME_POS - 8, r1.bottom - TRAILER);
	if (r.Contains(thePoint)) {
		fWindow->PostMessage(KILL_TITLE_SAVE);
		index = (thePoint.y - HEADER) / CELL_HEIGHT;
		if (fTextView)
			KillEditField(TRUE);
		OpenEditField(index);
	}
}

//--------------------------------------------------------------------

void TTrackView::Pulse()
{
	long				position;
	BRect				r;
	scsi_position		pos;

	if (fCDID) {
		ioctl(fCDID, SCSI_GET_POSITION, &pos);
		if ((pos.position[1] == 0x11) || (pos.position[1] == 0x12)) {
			if ((fTrack) && (fTrack != pos.position[6]))
				DrawTrack(fTrack, FALSE);
			if ((pos.position[6]) && (fTrack != pos.position[6]))
				DrawTrack(pos.position[6], TRUE);
		}
		else if (fTrack) {
			DrawTrack(fTrack, FALSE);
			fTrack = 0;
		}
	}
}

//--------------------------------------------------------------------

void TTrackView::DrawTrack(long track, bool selected)
{
	char		string[256];
	long		length;
	BRect		r;
	track_info	*track_data;

	track_data = (track_info*)fList->ItemAt(track);
	r.Set(MARGIN + 1, HEADER + ((track - 1) * CELL_HEIGHT) + 1,
		  MARGIN + CELL_WIDTH - 1, HEADER + (track * CELL_HEIGHT) - 2);
	if (selected) {
		fTrack = track;
		SetHighColor(136, 136, 136);
	}
	else
		SetHighColor(216, 216, 216);
	FillRect(r);
	SetDrawingMode(B_OP_OVER);
	SetHighColor(0, 0, 0);
	MovePenTo(TRACK_POS, HEADER + track * CELL_HEIGHT - 5);
	sprintf(string, "%d", track);
	DrawString(string);
	MovePenTo(TITLE_POS, HEADER + track * CELL_HEIGHT - 5);
	strcpy(string, track_data->title);
	while (StringWidth(string) > (TIME_POS - TITLE_POS) - 8) {
		string[strlen(string) - 2] = 0xc9;
		string[strlen(string) - 1] = 0;
	}
	DrawString(string);
	length = (fTOC->toc_data[4 + (track * 8) + 5] * 60) +
			 (fTOC->toc_data[4 + (track * 8) + 6]);
	length -= ((fTOC->toc_data[4 + ((track - 1) * 8) + 5] * 60) +
			   (fTOC->toc_data[4 + ((track - 1) * 8) + 6]));
	sprintf(string, "%.2d:%.2d", length / 60, length % 60);
	MovePenTo(TIME_POS, HEADER + track * CELL_HEIGHT - 5);
	DrawString(string);
	SetDrawingMode(B_OP_COPY);
}

//--------------------------------------------------------------------

void TTrackView::KillEditField(bool save)
{
	const char	*text;
	long		length;
	long		loop;
	track_info	*track_data;

	if (fTextView) {
		RemoveChild(fTextView);
		if (save) {
			track_data = (track_info*)(fList->ItemAt(fField + 1));
			if (track_data) {
				text = fTextView->Text();
				length = fTextView->TextLength();
				for (loop = 0; loop < length; loop++)
					track_data->title[loop] = *text++;
				track_data->title[loop] = 0;
				track_data->flags |= DIRTY;
				fWindow->PostMessage(SAVE_DATA);
			}
		}
		delete fTextView;
		fTextView = NIL;
	}
}

//--------------------------------------------------------------------

void TTrackView::NextField(bool prev)
{
	long	field;

	field = fField;
	if (fTextView)
		KillEditField(TRUE);
	if (prev) {
		field--;
		if (field < 0)
			field = fTracks - 1;
	}
	else {
		field++;
		if (field == fTracks)
			field = 0;
	}
	OpenEditField(field);
}

//--------------------------------------------------------------------

void TTrackView::OpenEditField(long field)
{
	BRect		r;
	BRect		dr;
	track_info	*track_data;

	fField = field;
	r.Set(TITLE_POS - 1, HEADER + (CELL_HEIGHT * fField) + 1,
		  TIME_POS - 8, HEADER + (CELL_HEIGHT * (fField + 1) - 3));
	dr = r;
	dr.OffsetTo(0, 0);
	fTextView = new TEditTextView(r, "", dr, B_FOLLOW_NONE, 0, "Erich", 9);
	AddChild(fTextView);
	track_data = (track_info*)(fList->ItemAt(fField + 1));
	if (track_data)
		fTextView->SetText(track_data->title);
	fTextView->SelectAll();
	fTextView->SetMaxChars(sizeof(track_data->title) - 1);
}
